'use strict';

exports.__esModule = true;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

var _actionTypes = require('./actionTypes');

var _mapValues = require('./mapValues');

var _mapValues2 = _interopRequireDefault(_mapValues);

var initialState = {
  asyncErrors: { valid: true },
  asyncValidating: false,
  data: {},
  initial: {},
  submitting: false,
  touched: {}
};

exports.initialState = initialState;
var reducer = function reducer() {
  var state = arguments.length <= 0 || arguments[0] === undefined ? initialState : arguments[0];

  var _extends2, _extends4;

  var action = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

  switch (action.type) {
    case _actionTypes.BLUR:
      var blurDiff = {
        data: _extends({}, state.data, (_extends2 = {}, _extends2[action.field] = action.value, _extends2))
      };
      if (action.touch) {
        var _extends3;

        blurDiff.touched = _extends({}, state.touched, (_extends3 = {}, _extends3[action.field] = true, _extends3));
      }
      return _extends({}, state, blurDiff);
    case _actionTypes.CHANGE:
      var _state$asyncErrors = state.asyncErrors,
          oldError = _state$asyncErrors[action.field],
          valid = _state$asyncErrors.valid,
          otherErrors = _objectWithoutProperties(_state$asyncErrors, [action.field, 'valid']);

      var changeDiff = {
        data: _extends({}, state.data, (_extends4 = {}, _extends4[action.field] = action.value, _extends4)),
        asyncErrors: _extends({}, otherErrors, {
          valid: !Object.keys(otherErrors).length
        })
      };
      delete changeDiff.asyncErrors[action.field];

      if (action.touch) {
        var _extends5;

        changeDiff.touched = _extends({}, state.touched, (_extends5 = {}, _extends5[action.field] = true, _extends5));
      }
      return _extends({}, state, changeDiff);
    case _actionTypes.INITIALIZE:
      return {
        asyncErrors: {},
        asyncValidating: false,
        data: action.data,
        initial: action.data,
        submitting: false,
        touched: {}
      };
    case _actionTypes.RESET:
      return {
        asyncErrors: {},
        asyncValidating: false,
        data: state.initial,
        initial: state.initial,
        submitting: false,
        touched: {}
      };
    case _actionTypes.START_ASYNC_VALIDATION:
      return _extends({}, state, {
        asyncValidating: true
      });
    case _actionTypes.START_SUBMIT:
      return _extends({}, state, {
        submitting: true
      });
    case _actionTypes.STOP_ASYNC_VALIDATION:
      return _extends({}, state, {
        asyncValidating: false,
        asyncErrors: action.errors
      });
    case _actionTypes.STOP_SUBMIT:
      return _extends({}, state, {
        submitting: false
      });
    case _actionTypes.TOUCH:
      var touchDiff = {};
      action.fields.forEach(function (field) {
        if (typeof field !== 'string') {
          throw new Error('fields passed to touch() must be strings');
        }
        touchDiff[field] = true;
      });
      return _extends({}, state, {
        touched: _extends({}, state.touched, touchDiff)
      });
    case _actionTypes.UNTOUCH:
      var untouchDiff = {};
      action.fields.forEach(function (field) {
        if (typeof field !== 'string') {
          throw new Error('fields passed to untouch() must be strings');
        }
        untouchDiff[field] = false;
      });
      return _extends({}, state, {
        touched: _extends({}, state.touched, untouchDiff)
      });
    default:
      return state;
  }
};

function formReducer() {
  var state = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

  var _extends8;

  var action = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];
  var form = action.form;
  var key = action.key;

  var rest = _objectWithoutProperties(action, ['form', 'key']);

  if (!form) {
    return state;
  }
  if (key) {
    var _extends6, _extends7;

    return _extends({}, state, (_extends7 = {}, _extends7[form] = _extends({}, state[form], (_extends6 = {}, _extends6[key] = reducer((state[form] || {})[key], rest), _extends6)), _extends7));
  }
  return _extends({}, state, (_extends8 = {}, _extends8[form] = reducer(state[form], rest), _extends8));
}

formReducer.plugin = function (reducers) {
  return function () {
    var state = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];
    var action = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

    var result = formReducer(state, action);
    return _extends({}, result, _mapValues2['default'](reducers, function (red, key) {
      return red(result[key] || initialState, action);
    }));
  };
};

exports['default'] = formReducer;