'use strict';

exports.__esModule = true;

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ('value' in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports['default'] = reduxForm;

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj['default'] = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } }

function _inherits(subClass, superClass) { if (typeof superClass !== 'function' && superClass !== null) { throw new TypeError('Super expression must either be null or a function, not ' + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _actions = require('./actions');

var formActions = _interopRequireWildcard(_actions);

var _util = require('./util');

var _bindActionData = require('./bindActionData');

var _bindActionData2 = _interopRequireDefault(_bindActionData);

var _reducer = require('./reducer');

function getSubForm(form, formName, formKey) {
  if (form && form[formName]) {
    if (formKey) {
      if (form[formName][formKey]) {
        return form[formName][formKey];
      }
    } else {
      return form[formName];
    }
  }
  return _reducer.initialState;
}

function getValue(passedValue, event) {
  if (passedValue !== undefined || !event) {
    return passedValue;
  }
  var _event$target = event.target;
  var type = _event$target.type;
  var value = _event$target.value;
  var checked = _event$target.checked;

  if (type === 'checkbox') {
    return checked;
  }
  return value;
}

function createReduxFormDecorator(formName, fields, syncValidate, touchOnBlur, touchOnChange, asyncValidate, asyncBlurFields) {
  function combineValidationErrors(form) {
    var syncErrors = syncValidate(form.data);
    var asyncErrors = _extends({ valid: true }, form.asyncErrors);
    var valid = !!(syncErrors.valid && asyncErrors.valid); // !! to convert falsy to boolean
    return _extends({}, syncErrors, asyncErrors, {
      valid: valid
    });
  }

  return function (DecoratedComponent) {
    return (function (_Component) {
      _inherits(ReduxForm, _Component);

      function ReduxForm() {
        _classCallCheck(this, ReduxForm);

        _Component.apply(this, arguments);
      }

      ReduxForm.prototype.render = function render() {
        var _this = this;

        var _props = this.props;
        var formName = _props.formName;
        var form = _props.form;
        var formKey = _props.formKey;
        var dispatch = _props.dispatch;

        var passableProps = _objectWithoutProperties(_props, ['formName', 'form', 'formKey', 'dispatch']);

        // eslint-disable-line no-shadow
        var subForm = getSubForm(form, formName, formKey);

        var _ref = formKey ? _bindActionData2['default'](formActions, { form: formName, key: formKey }) : _bindActionData2['default'](formActions, { form: formName });

        var blur = _ref.blur;
        var change = _ref.change;
        var initialize = _ref.initialize;
        var reset = _ref.reset;
        var startAsyncValidation = _ref.startAsyncValidation;
        var startSubmit = _ref.startSubmit;
        var stopAsyncValidation = _ref.stopAsyncValidation;
        var stopSubmit = _ref.stopSubmit;
        var touch = _ref.touch;
        var untouch = _ref.untouch;

        var runAsyncValidation = asyncValidate ? function () {
          dispatch(startAsyncValidation(formKey));
          var promise = asyncValidate(subForm.data);
          if (!promise || typeof promise.then !== 'function') {
            throw new Error('asyncValidate function passed to reduxForm must return a promise!');
          }
          return promise.then(function (asyncErrors) {
            dispatch(stopAsyncValidation(asyncErrors));
            return !!asyncErrors.valid;
          });
        } : undefined;
        var handleBlur = function handleBlur(name, value) {
          return function (event) {
            var fieldValue = getValue(value, event);
            var doBlur = _bindActionData2['default'](blur, { touch: touchOnBlur });
            dispatch(doBlur(name, fieldValue));
            if (runAsyncValidation && ~asyncBlurFields.indexOf(name)) {
              var _extends2;

              var syncError = syncValidate(_extends({}, subForm.data, (_extends2 = {}, _extends2[name] = fieldValue, _extends2)))[name];
              // only dispatch async call if all synchronous client-side validation passes for this field
              if (!syncError) {
                runAsyncValidation();
              }
            }
          };
        };
        var pristine = _util.isPristine(subForm.initial, subForm.data);

        var _combineValidationErrors = combineValidationErrors(subForm);

        var valid = _combineValidationErrors.valid;

        var errors = _objectWithoutProperties(_combineValidationErrors, ['valid']);

        var handleChange = function handleChange(name, value) {
          return function (event) {
            var doChange = _bindActionData2['default'](change, { touch: touchOnChange });
            dispatch(doChange(name, getValue(value, event)));
          };
        };
        var handleSubmit = function handleSubmit(submitOrEvent) {
          var createEventHandler = function createEventHandler(submit) {
            return function (event) {
              if (event) {
                event.preventDefault();
              }
              var submitWithPromiseCheck = function submitWithPromiseCheck() {
                var result = submit(subForm.data);
                if (result && typeof result.then === 'function') {
                  // you're showing real promise, kid!
                  var stopAndReturn = function stopAndReturn(x) {
                    dispatch(stopSubmit());
                    return x;
                  };
                  dispatch(startSubmit());
                  result.then(stopAndReturn, stopAndReturn);
                }
              };
              dispatch(touch.apply(undefined, fields));
              if (runAsyncValidation) {
                return runAsyncValidation().then(function (asyncValid) {
                  if (valid && asyncValid) {
                    return submitWithPromiseCheck(subForm.data);
                  }
                });
              } else if (valid) {
                return submitWithPromiseCheck(subForm.data);
              }
            };
          };
          if (typeof submitOrEvent === 'function') {
            return createEventHandler(submitOrEvent);
          }
          var onSubmit = _this.props.onSubmit;

          if (!onSubmit) {
            throw new Error('You must either pass handleSubmit() an onSubmit function or pass onSubmit as a prop');
          }
          createEventHandler(onSubmit)(submitOrEvent /* is event */);
        };
        return _react2['default'].createElement(DecoratedComponent, _extends({
          asyncValidate: runAsyncValidation,
          asyncValidating: subForm.asyncValidating,
          data: subForm.data,
          dirty: !pristine,
          dispatch: dispatch,
          errors: errors,
          formKey: formKey,
          handleBlur: handleBlur,
          handleChange: handleChange,
          handleSubmit: handleSubmit,
          initializeForm: function (data) {
            return dispatch(initialize(data));
          },
          invalid: !valid,
          pristine: pristine,
          resetForm: function () {
            return dispatch(reset());
          },
          submitting: subForm.submitting,
          touch: function () {
            for (var _len = arguments.length, touchFields = Array(_len), _key = 0; _key < _len; _key++) {
              touchFields[_key] = arguments[_key];
            }

            return dispatch(touch.apply(undefined, touchFields));
          },
          touched: subForm.touched,
          touchAll: function () {
            return dispatch(touch.apply(undefined, fields));
          },
          untouch: function () {
            for (var _len2 = arguments.length, untouchFields = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
              untouchFields[_key2] = arguments[_key2];
            }

            return dispatch(untouch.apply(undefined, untouchFields));
          },
          untouchAll: function () {
            return dispatch(untouchAll.apply(undefined, fields));
          },
          valid: valid
        }, passableProps)); // pass other props
      };

      _createClass(ReduxForm, null, [{
        key: 'displayName',
        value: 'ReduxForm(' + _util.getDisplayName(DecoratedComponent) + ')',
        enumerable: true
      }, {
        key: 'DecoratedComponent',
        value: DecoratedComponent,
        enumerable: true
      }, {
        key: 'propTypes',
        value: {
          formName: _react.PropTypes.string,
          formKey: _react.PropTypes.string,
          form: _react.PropTypes.object,
          onSubmit: _react.PropTypes.func,
          dispatch: _react.PropTypes.func.isRequired
        },
        enumerable: true
      }, {
        key: 'defaultProps',
        value: {
          formName: formName
        },
        enumerable: true
      }]);

      return ReduxForm;
    })(_react.Component);
  };
}

function reduxForm(formName, fields) {
  var syncValidate = arguments.length <= 2 || arguments[2] === undefined ? function () {
    return { valid: true };
  } : arguments[2];
  var touchOnBlur = arguments.length <= 3 || arguments[3] === undefined ? true : arguments[3];
  var touchOnChange = arguments.length <= 4 || arguments[4] === undefined ? false : arguments[4];

  var decorator = createReduxFormDecorator(formName, fields, syncValidate, !!touchOnBlur, !!touchOnChange);
  decorator.async = function (asyncValidate) {
    for (var _len3 = arguments.length, blurFields = Array(_len3 > 1 ? _len3 - 1 : 0), _key3 = 1; _key3 < _len3; _key3++) {
      blurFields[_key3 - 1] = arguments[_key3];
    }

    return createReduxFormDecorator(formName, fields, syncValidate, !!touchOnBlur, !!touchOnChange, asyncValidate, Array.isArray(blurFields[0]) ? blurFields[0] : blurFields);
  };
  return decorator;
}

module.exports = exports['default'];